name: {{ package_name }}
description: Auto-generated Dart HTTP client for the {{ spec.name }} service.
version: {{ client_version }}

environment:
  sdk: '>=3.0.0 <4.0.0'

dependencies:
  http: ^1.1.0
