"""Generated client for {{ spec.name }}."""

from {{ package_name }}.client import {{ class_name }}
{% if versions %}
{% for version in versions %}
from {{ package_name }} import {{ version }}
{% endfor %}
{% endif %}
{% if schemas %}
from {{ package_name }} import schemas
{% endif %}

__all__ = [
    "{{ class_name }}",
{% if versions %}
{% for version in versions %}
    "{{ version }}",
{% endfor %}
{% endif %}
{% if schemas %}
    "schemas",
{% endif %}
]
