# {{ go_module }}

Auto-generated Go HTTP client for the **{{ spec.name }}** service.

Generated by [pyramid-client-builder](https://pypi.org/project/pyramid-client-builder/).

## Installation

```
go get {{ go_module }}
```

## Usage

```go
import "{{ go_module }}"

client := {{ go_package }}.NewClient("https://api.example.com")
```
{% if versions %}

### Versioned endpoints

```go
{% for version in versions %}
// {{ version }} endpoints
result, err := client.{{ version | go_version_field }}.SomeMethod(...)
{% endfor %}
```
{% endif %}
