[project]
name = "{{ project_name }}"
version = "{{ client_version }}"
description = "Auto-generated HTTP client for {{ spec.name }}."
requires-python = ">=3.10"
dependencies = [
{% if http_client == "httpx" %}
    "httpx>=0.24",
{% else %}
    "requests>=2.28",
{% endif %}
{% if has_schemas %}
    "marshmallow>=3.0",
{% endif %}
]

[project.optional-dependencies]
pyramid = ["pyramid"]

[build-system]
requires = ["hatchling"]
build-backend = "hatchling.build"

[tool.hatch.build.targets.wheel]
packages = ["{{ package_name }}"]
