"""pyramid-client-builder: Generate HTTP clients from Pyramid applications."""

from pyramid_client_builder.generator.core import ClientGenerator
from pyramid_client_builder.introspection import PyramidIntrospector
from pyramid_client_builder.models import ClientSpec, EndpointInfo
from pyramid_client_builder.version import __version__

__all__ = [
    "ClientGenerator",
    "ClientSpec",
    "EndpointInfo",
    "PyramidIntrospector",
    "__version__",
]
