# {{ project_name }}

Auto-generated HTTP client for the **{{ spec.name }}** service.

Generated by [pyramid-client-builder](https://pypi.org/project/pyramid-client-builder/).

## Installation

```
pip install {{ project_name }}
```

## Usage

```python
from {{ package_name }} import {{ class_name }}

client = {{ class_name }}(base_url="https://api.example.com")
```

### Pyramid integration

Add to your INI configuration:

```ini
{{ spec.settings_prefix }}.base_url = https://api.example.com
```

Include in your Pyramid app:

```python
config.include("{{ package_name }}.ext")
```

Access via the request:

```python
def my_view(request):
    result = request.{{ request_attr }}.some_method()
```
