# {{ package_name }}

Auto-generated Dart HTTP client for the **{{ spec.name }}** service.

Generated by [pyramid-client-builder](https://pypi.org/project/pyramid-client-builder/).

## Installation

Add to your `pubspec.yaml`:

```yaml
dependencies:
  {{ package_name }}:
    path: ./{{ package_name }}
```

## Usage

```dart
import 'package:{{ package_name }}/{{ package_name }}.dart';

final client = {{ class_name }}('https://api.example.com');
```
{% if versions %}

### Versioned endpoints

```dart
{% for version in versions %}
// {{ version }} endpoints
final result = await client.{{ version | dart_version_field }}.someMethod(...);
{% endfor %}
```
{% endif %}
